@@ -2,18 +2,20 @@
 // Copyright 2021 Keylime Authors
 
 use crate::{
-    common::JsonWrapper,
-    config::KeylimeConfig,
+    config::AgentConfig,
     payloads::{Payload, PayloadMessage},
     Error, QuoteData, Result,
 };
 use actix_web::{http, web, HttpRequest, HttpResponse, Responder};
 use base64::{engine::general_purpose, Engine as _};
-use keylime::crypto::{
-    self,
-    auth_tag::AuthTag,
-    encrypted_data::EncryptedData,
-    symmkey::{KeySet, SymmKey},
+use keylime::{
+    crypto::{
+        self,
+        auth_tag::AuthTag,
+        encrypted_data::EncryptedData,
+        symmkey::{KeySet, SymmKey},
+    },
+    json_wrapper::JsonWrapper,
 };
 use log::*;
 use serde::{Deserialize, Serialize};
@@ -329,7 +331,7 @@ async fn pubkey(
             HttpResponse::Ok().json(response)
         }
         Err(e) => {
-            debug!("Unable to retrieve public key: {:?}", e);
+            debug!("Unable to retrieve public key: {e:?}");
             HttpResponse::InternalServerError().json(JsonWrapper::error(
                 500,
                 "Unable to retrieve public key".to_string(),
@@ -413,7 +415,7 @@ async fn verify(
                 HttpResponse::Ok().json(response)
             }
             Err(e) => {
-                warn!("GET key challenge failed: {:?}", e);
+                warn!("GET key challenge failed: {e:?}");
                 HttpResponse::InternalServerError().json(JsonWrapper::error(
                     500,
                     "GET key challenge failed".to_string(),
@@ -605,7 +607,6 @@ mod tests {
         encrypt_aead, pkey_pub_from_pem, rsa_oaep_encrypt,
     };
     use crate::{
-        config::KeylimeConfig,
         crypto::{compute_hmac, AES_128_KEY_LEN, AES_256_KEY_LEN},
         payloads,
     };
@@ -626,6 +627,9 @@ mod tests {
     };
     use tokio::sync::mpsc;
 
+    #[cfg(feature = "testing")]
+    use crate::config::get_testing_config;
+
     // Enough length for testing both AES-128 and AES-256
     const U: &[u8; AES_256_KEY_LEN] = b"01234567890123456789012345678901";
     const V: &[u8; AES_256_KEY_LEN] = b"ABCDEFGHIJABCDEFGHIJABCDEFGHIJAB";
@@ -875,11 +879,12 @@ mod tests {
 
     #[cfg(feature = "testing")]
     async fn test_u_or_v_key(key_len: usize, payload: Option<&[u8]>) {
-        let test_config = KeylimeConfig::default();
-        let (mut fixture, mutex) = QuoteData::fixture().await.unwrap(); //#[allow_ci]
-
         // Create temporary working directory and secure mount
         let temp_workdir = tempfile::tempdir().unwrap(); //#[allow_ci]
+        let test_config = get_testing_config(temp_workdir.path());
+
+        let (mut fixture, mutex) = QuoteData::fixture().await.unwrap(); //#[allow_ci]
+
         fixture.secure_mount =
             PathBuf::from(&temp_workdir.path().join("tmpfs-dev"));
         fs::create_dir(&fixture.secure_mount).unwrap(); //#[allow_ci]
@@ -918,7 +923,7 @@ mod tests {
             encrypt_aead(k.as_ref(), &iv[..], payload).unwrap() //#[allow_ci]
         });
 
-        let uuid = test_config.agent.uuid;
+        let uuid = test_config.uuid;
         let auth_tag = compute_hmac(k.as_ref(), uuid.as_bytes()).unwrap(); //#[allow_ci]
 
         let arbiter = Arbiter::new();
@@ -929,7 +934,7 @@ mod tests {
             let result = worker(true, uuid_clone, keys_rx, p_tx).await;
 
             if result.is_err() {
-                debug!("keys worker failed: {:?}", result);
+                debug!("keys worker failed: {result:?}");
             }
         })));
 
