@@ -2,7 +2,6 @@
 // Copyright 2021 Keylime Authors
 
 use crate::{
-    config,
     revocation::{Revocation, RevocationMessage},
     Error, Result,
 };
@@ -10,12 +9,15 @@ use crate::{
 #[cfg(feature = "with-zmq")]
 use crate::revocation::ZmqMessage;
 
-use keylime::crypto::{
-    self,
-    encrypted_data::EncryptedData,
-    symmkey::{KeySet, SymmKey},
+use keylime::{
+    config,
+    crypto::{
+        self,
+        encrypted_data::EncryptedData,
+        symmkey::{KeySet, SymmKey},
+    },
+    permissions,
 };
-use keylime::global_config;
 use log::*;
 use serde::{Deserialize, Serialize};
 use serde_json::json;
@@ -70,7 +72,7 @@ fn decrypt_payload(
 // writing out symmetric key and encrypted payload. returns file paths for
 // both.
 fn setup_unzipped(
-    config: &config::KeylimeConfig,
+    config: &config::AgentConfig,
     mount: &Path,
 ) -> Result<(PathBuf, PathBuf, PathBuf)> {
     let unzipped = mount.join("unzipped");
@@ -80,15 +82,15 @@ fn setup_unzipped(
         fs::remove_dir_all(&unzipped)?;
     }
 
-    match config.agent.dec_payload_file.as_ref() {
-        "" => Err(global_config::KeylimeConfigError::RequiredOption(
+    match config.dec_payload_file.as_ref() {
+        "" => Err(config::KeylimeConfigError::RequiredOption(
             "dec_payload_path".to_string(),
         )
         .into()),
         p => {
             let dec_payload_path = unzipped.join(p);
-            match config.agent.enc_keyname.as_ref() {
-                "" => Err(global_config::KeylimeConfigError::RequiredOption(
+            match config.enc_keyname.as_ref() {
+                "" => Err(config::KeylimeConfigError::RequiredOption(
                     "enc_keyname".to_string(),
                 )
                 .into()),
@@ -114,14 +116,14 @@ fn write_out_key_and_payload(
     if bytes != key.as_ref().len() {
         return Err(Error::Other(format!("Error writing symm key to {:?}: key len is {}, but {bytes} bytes were written", key_path, key.as_ref().len())));
     }
-    info!("Wrote payload decryption key to {:?}", key_path);
+    info!("Wrote payload decryption key to {key_path:?}");
 
     let mut dec_payload_file = fs::File::create(dec_payload_path)?;
     let bytes = dec_payload_file.write(dec_payload)?;
     if bytes != dec_payload.len() {
         return Err(Error::Other(format!("Error writing decrypted payload to {:?}: payload len is {}, but {bytes} bytes were written", dec_payload_path, dec_payload.len())));
     }
-    info!("Wrote decrypted payload to {:?}", dec_payload_path);
+    info!("Wrote decrypted payload to {dec_payload_path:?}");
 
     Ok(())
 }
@@ -129,7 +131,7 @@ fn write_out_key_and_payload(
 // run a script (such as the init script, if any) and check the status
 fn run(dir: &Path, script: &str) -> Result<()> {
     let script_path = dir.join(script);
-    info!("Running script: {:?}", script_path);
+    info!("Running script: {script_path:?}");
 
     if !script_path.exists() {
         info!("No payload script {script} found in {}", dir.display());
@@ -171,17 +173,17 @@ fn run(dir: &Path, script: &str) -> Result<()> {
 // the input string is the directory where the unzipped file(s) should be stored.
 fn optional_unzip_payload(
     unzipped: &Path,
-    config: &config::KeylimeConfig,
+    config: &config::AgentConfig,
 ) -> Result<()> {
-    if config.agent.extract_payload_zip {
-        match config.agent.dec_payload_file.as_ref() {
+    if config.extract_payload_zip {
+        match config.dec_payload_file.as_ref() {
             "" => {
                 warn!("Configuration option dec_payload_file not set, not unzipping payload");
             }
             dec_file => {
                 let zipped_payload_path = unzipped.join(dec_file);
 
-                info!("Unzipping payload {} to {:?}", dec_file, unzipped);
+                info!("Unzipping payload {dec_file} to {unzipped:?}");
 
                 let mut source = fs::File::open(zipped_payload_path)?;
                 let mut zip = ZipArchive::new(source)?;
@@ -196,7 +198,7 @@ fn optional_unzip_payload(
 async fn run_encrypted_payload(
     symm_key: SymmKey,
     payload: EncryptedData,
-    config: &config::KeylimeConfig,
+    config: &config::AgentConfig,
     mount: &Path,
     revocation_tx: Sender<RevocationMessage>,
     #[cfg(feature = "with-zmq")] zmq_tx: Sender<ZmqMessage>,
@@ -215,12 +217,12 @@ async fn run_encrypted_payload(
 
     optional_unzip_payload(&unzipped, config)?;
     // there may also be also a separate init script
-    match config.agent.payload_script.as_ref() {
+    match config.payload_script.as_ref() {
         "" => {
             info!("No payload script specified, skipping");
         }
         script => {
-            info!("Payload init script indicated: {}", script);
+            info!("Payload init script indicated: {script}");
             run(&unzipped, script)?;
         }
     }
@@ -239,20 +241,21 @@ async fn run_encrypted_payload(
             .map(|script| unzipped.join(script))
             .filter(|script| script.exists())
             .try_for_each(|script| {
-                if fs::set_permissions(
-                    &script,
-                    fs::Permissions::from_mode(0o700),
-                )
-                .is_err()
-                {
-                    error!(
-                        "Could not set permission for action {}",
-                        script.display()
-                    );
-                    Err(Error::Permission)
-                } else {
-                    info!("Permission set for action: {}", script.display());
-                    Ok(())
+                match permissions::set_mode(&script, 0o700) {
+                    Ok(()) => {
+                        info!(
+                            "Permission set for action: {}",
+                            script.display()
+                        );
+                        Ok(())
+                    }
+                    Err(e) => {
+                        error!(
+                            "Could not set permission for action {}",
+                            script.display()
+                        );
+                        Err(e)
+                    }
                 }
             })?
     }
@@ -277,7 +280,7 @@ async fn run_encrypted_payload(
 }
 
 pub(crate) async fn worker(
-    config: config::KeylimeConfig,
+    config: config::AgentConfig,
     mount: impl AsRef<Path>,
     mut payload_rx: Receiver<PayloadMessage>,
     mut revocation_tx: Sender<RevocationMessage>,
@@ -309,7 +312,7 @@ pub(crate) async fn worker(
                         info!("Successfully executed encrypted payload");
                     }
                     Err(e) => {
-                        warn!("Failed to run encrypted payload: {}", e);
+                        warn!("Failed to run encrypted payload: {e:?}");
                     }
                 }
             }
@@ -324,22 +327,32 @@ pub(crate) async fn worker(
 #[cfg(test)]
 mod tests {
     use super::*;
-    #[cfg(feature = "testing")]
-    use crate::crypto::testing::{
-        encrypt_aead, pkey_pub_from_pem, rsa_oaep_encrypt,
-    };
     use crate::{
-        config::KeylimeConfig,
         crypto::{AES_128_KEY_LEN, AES_256_KEY_LEN},
         payloads,
     };
     use actix_rt::Arbiter;
+    use keylime::config::AgentConfig;
     use std::{
         env, fs,
         path::{Path, PathBuf},
     };
     use tokio::sync::mpsc;
 
+    #[cfg(feature = "testing")]
+    use crate::crypto::testing::{
+        encrypt_aead, pkey_pub_from_pem, rsa_oaep_encrypt,
+    };
+    #[cfg(feature = "testing")]
+    use keylime::config::get_testing_config;
+    #[cfg(feature = "testing")]
+    use std::sync::OnceLock;
+    #[cfg(feature = "testing")]
+    use tokio::sync::Mutex as AsyncMutex;
+
+    #[cfg(feature = "testing")]
+    pub static MUTEX: OnceLock<Arc<AsyncMutex<()>>> = OnceLock::new();
+
     // Enough length for testing both AES-128 and AES-256
     const U: &[u8; AES_256_KEY_LEN] = b"01234567890123456789012345678901";
     const V: &[u8; AES_256_KEY_LEN] = b"ABCDEFGHIJABCDEFGHIJABCDEFGHIJAB";
@@ -398,10 +411,11 @@ echo hello > test-output
         assert!(result.is_ok());
     }
 
+    #[cfg(feature = "testing")]
     #[test]
     fn test_setup_unzipped() {
-        let test_config = KeylimeConfig::default();
         let temp_workdir = tempfile::tempdir().unwrap(); //#[allow_ci]
+        let test_config = get_testing_config(temp_workdir.path(), None);
         let secure_mount =
             PathBuf::from(&temp_workdir.path().join("tmpfs-dev"));
         fs::create_dir(&secure_mount).unwrap(); //#[allow_ci]
@@ -410,10 +424,9 @@ echo hello > test-output
         let (unzipped, dec_payload_path, key_path) = result.unwrap(); //#[allow_ci]
         assert!(unzipped.exists());
         assert!(
-            dec_payload_path
-                == unzipped.join(test_config.agent.dec_payload_file)
+            dec_payload_path == unzipped.join(test_config.dec_payload_file)
         );
-        assert!(key_path == unzipped.join(test_config.agent.enc_keyname));
+        assert!(key_path == unzipped.join(test_config.enc_keyname));
     }
 
     #[test]
@@ -431,19 +444,19 @@ echo hello > test-output
         assert!(result.is_ok());
     }
 
+    #[cfg(feature = "testing")]
     #[test]
     fn test_unzip_payload() {
-        let mut test_config = KeylimeConfig::default();
         let temp_workdir = tempfile::tempdir().unwrap(); //#[allow_ci]
+        let test_config = get_testing_config(temp_workdir.path(), None);
         let payload_path = Path::new(env!("CARGO_MANIFEST_DIR"))
             .join("test-data")
             .join("payload.zip");
 
-        let dec_payload_file =
-            match test_config.agent.dec_payload_file.as_ref() {
-                "" => panic!("dec_payload_file not set by default"), //#[allow_ci]
-                f => f,
-            };
+        let dec_payload_file = match test_config.dec_payload_file.as_ref() {
+            "" => panic!("dec_payload_file not set by default"), //#[allow_ci]
+            f => f,
+        };
 
         let result = fs::copy(
             payload_path,
@@ -463,8 +476,12 @@ echo hello > test-output
     #[cfg(feature = "testing")]
     #[actix_rt::test]
     async fn test_run_encrypted_payload() {
-        let test_config = KeylimeConfig::default();
+        let _mutex = MUTEX
+            .get_or_init(|| Arc::new(AsyncMutex::new(())))
+            .lock()
+            .await;
         let temp_workdir = tempfile::tempdir().unwrap(); //#[allow_ci]
+        let test_config = get_testing_config(temp_workdir.path(), None);
         let secure_mount =
             PathBuf::from(&temp_workdir.path().join("tmpfs-dev"));
         fs::create_dir(&secure_mount).unwrap(); //#[allow_ci]
@@ -507,10 +524,14 @@ echo hello > test-output
     #[cfg(feature = "testing")]
     #[actix_rt::test]
     async fn test_payload_worker() {
+        let _mutex = MUTEX
+            .get_or_init(|| Arc::new(AsyncMutex::new(())))
+            .lock()
+            .await;
         use crate::{config::DEFAULT_PAYLOAD_SCRIPT, secure_mount};
 
-        let test_config = KeylimeConfig::default();
         let temp_workdir = tempfile::tempdir().unwrap(); //#[allow_ci]
+        let test_config = get_testing_config(temp_workdir.path(), None);
         let secure_mount =
             PathBuf::from(&temp_workdir.path().join("tmpfs-dev"));
         fs::create_dir(&secure_mount).unwrap(); //#[allow_ci]
@@ -544,7 +565,7 @@ echo hello > test-output
             .await;
 
             if result.is_err() {
-                debug!("payloads worker failed: {:?}", result);
+                debug!("payloads worker failed: {result:?}");
             }
 
             let timestamp_path = temp_workdir.path().join("timestamp");
