@@ -3,11 +3,10 @@
 
 #[macro_use]
 use actix_web::rt;
-use crate::config::{AgentConfig, KeylimeConfig};
+use crate::config::AgentConfig;
 use crate::crypto;
 use crate::secure_mount;
-use keylime::keylime_error::*;
-use keylime::list_parser::parse_list;
+use keylime::{error::*, list_parser::parse_list};
 use log::*;
 use serde::{Deserialize, Serialize};
 use serde_json::Value;
@@ -124,7 +123,7 @@ pub(crate) fn run_action(
         allow_payload_actions,
     )?;
 
-    info!("Executing revocation action {}", action);
+    info!("Executing revocation action {action}");
 
     // Write JSON argument to a temporary file
     let raw_json = serde_json::value::to_raw_value(&json)?;
@@ -171,7 +170,7 @@ pub(crate) fn run_action(
         return Err(output.try_into()?);
     }
 
-    info!("INFO: revocation action {} successful", action);
+    info!("INFO: revocation action {action} successful");
 
     Ok(output)
 }
@@ -233,7 +232,7 @@ fn run_revocation_actions(
                     let msg = format!(
                         "error executing revocation script {action}: {e:?}"
                     );
-                    error!("{}", msg);
+                    error!("{msg}");
                     return Err(Error::Script(
                         action.to_string(),
                         e.exe_code()?,
@@ -273,8 +272,7 @@ fn process_revocation(
         let msg_payload: Value = serde_json::from_str(msg)?;
 
         debug!(
-            "Revocation signature validated for revocation: {}",
-            msg_payload
+            "Revocation signature validated for revocation: {msg_payload}"
         );
 
         let outputs = run_revocation_actions(
@@ -289,11 +287,11 @@ fn process_revocation(
         for output in outputs {
             if !output.stdout.is_empty() {
                 let out = String::from_utf8(output.stdout)?;
-                info!("Action stdout: {}", out);
+                info!("Action stdout: {out}");
             }
             if !output.stderr.is_empty() {
                 let out = String::from_utf8(output.stderr)?;
-                warn!("Action stderr: {}", out);
+                warn!("Action stderr: {out}");
             }
         }
         Ok(())
@@ -318,14 +316,11 @@ fn listen_zmq(
 
     let endpoint = format!("tcp://{ip}:{port}");
 
-    info!(
-        "Connecting to revocation notification endpoint at {}...",
-        endpoint
-    );
+    info!("Connecting to revocation notification endpoint at {endpoint}...");
 
     mysock.connect(endpoint.as_str())?;
 
-    info!("Waiting for revocation messages on 0mq {}", endpoint);
+    info!("Waiting for revocation messages on 0mq {endpoint}");
 
     Ok(rt::spawn(async move {
         // Main revocation service loop. If a message is malformed or
@@ -381,7 +376,7 @@ fn listen_zmq(
             };
             sleep(Duration::from_millis(100)).await;
         }
-        info!("Stop waiting for revocation messages on 0mq {}", endpoint);
+        info!("Stop waiting for revocation messages on 0mq {endpoint}");
         Ok(())
     }))
 }
@@ -476,7 +471,7 @@ pub(crate) async fn worker(
                                 info!("Revocation processed successfully");
                             }
                             Err(e) => {
-                                error!("Failed to process revocation: {}", e);
+                                error!("Failed to process revocation: {e:?}");
                             }
                         }
                     }
@@ -522,9 +517,15 @@ mod tests {
     // Used to create symbolic links
     use std::os::unix::fs::symlink;
 
+    #[cfg(feature = "testing")]
+    use keylime::config::get_testing_config;
+
+    #[cfg(feature = "testing")]
     #[test]
     fn revocation_scripts_ok() {
-        let test_config = KeylimeConfig::default();
+        let work_dir = tempfile::tempdir()
+            .expect("failed to create temporary directory");
+        let test_config = get_testing_config(work_dir.path());
         let json_file = concat!(
             env!("CARGO_MANIFEST_DIR"),
             "/tests/unzipped/test_ok.json"
@@ -533,7 +534,6 @@ mod tests {
         let json = serde_json::from_str(&json_str).unwrap(); //#[allow_ci]
         let actions_dir =
             &Path::new(env!("CARGO_MANIFEST_DIR")).join("tests/actions/");
-        let work_dir = tempfile::tempdir().unwrap(); //#[allow_ci]
         let tmpfs_dir = work_dir.path().join("tmpfs-dev"); //#[allow_ci]
         fs::create_dir(&tmpfs_dir).unwrap(); //#[allow_ci]
         let unzipped_dir =
@@ -561,9 +561,12 @@ mod tests {
         }
     }
 
+    #[cfg(feature = "testing")]
     #[test]
     fn revocation_scripts_err() {
-        let test_config = KeylimeConfig::default();
+        let work_dir = tempfile::tempdir()
+            .expect("failed to create temporary directory");
+        let test_config = get_testing_config(work_dir.path());
         let json_file = concat!(
             env!("CARGO_MANIFEST_DIR"),
             "/tests/unzipped/test_err.json"
@@ -572,7 +575,6 @@ mod tests {
         let json = serde_json::from_str(&json_str).unwrap(); //#[allow_ci]
         let actions_dir =
             &Path::new(env!("CARGO_MANIFEST_DIR")).join("tests/actions/");
-        let work_dir = tempfile::tempdir().unwrap(); //#[allow_ci]
         let tmpfs_dir = work_dir.path().join("tmpfs-dev"); //#[allow_ci]
         fs::create_dir(&tmpfs_dir).unwrap(); //#[allow_ci]
         let unzipped_dir =
@@ -589,9 +591,12 @@ mod tests {
         assert!(outputs.is_err());
     }
 
+    #[cfg(feature = "testing")]
     #[test]
     fn revocation_scripts_from_config() {
-        let mut test_config = KeylimeConfig::default();
+        let work_dir = tempfile::tempdir()
+            .expect("failed to create temporary directory");
+        let test_config = get_testing_config(work_dir.path());
         let json_file = concat!(
             env!("CARGO_MANIFEST_DIR"),
             "/tests/unzipped/test_ok.json"
@@ -607,7 +612,6 @@ mod tests {
         let json = serde_json::from_str(&json_str).unwrap(); //#[allow_ci]
         let actions_dir =
             &Path::new(env!("CARGO_MANIFEST_DIR")).join("tests/actions/");
-        let work_dir = tempfile::tempdir().unwrap(); //#[allow_ci]
         let tmpfs_dir = work_dir.path().join("tmpfs-dev"); //#[allow_ci]
         fs::create_dir(&tmpfs_dir).unwrap(); //#[allow_ci]
         let unzipped_dir =
@@ -755,9 +759,12 @@ mod tests {
         ));
     }
 
+    #[cfg(feature = "testing")]
     #[test]
     fn test_process_revocation() {
-        let test_config = KeylimeConfig::default();
+        let work_dir = tempfile::tempdir()
+            .expect("failed to create temporary directory");
+        let test_config = get_testing_config(work_dir.path());
 
         let sig_path = Path::new(env!("CARGO_MANIFEST_DIR"))
             .join("test-data/revocation.sig");
@@ -777,16 +784,15 @@ mod tests {
         let actions_dir =
             Path::new(env!("CARGO_MANIFEST_DIR")).join("tests/actions");
 
-        let work_dir = Path::new(env!("CARGO_MANIFEST_DIR")).join("tests");
-        let tmpfs_dir = work_dir.join("tmpfs-dev");
+        let tmpfs_dir = work_dir.path().join("tmpfs-dev");
 
         let result = process_revocation(
             revocation,
             &cert,
             &actions_dir,
             None,
-            test_config.agent.allow_payload_revocation_actions,
-            &work_dir,
+            test_config.allow_payload_revocation_actions,
+            work_dir.path(),
             &tmpfs_dir,
         );
 
