@@ -185,6 +185,23 @@ pub fn get_key_size(tpm_encryption_alg: &EncryptionAlgorithm) -> usize {
     }
 }
 
+pub fn get_key_algorithm_family(
+    tpm_encryption_alg: &EncryptionAlgorithm,
+) -> &'static str {
+    match tpm_encryption_alg {
+        EncryptionAlgorithm::Rsa1024
+        | EncryptionAlgorithm::Rsa2048
+        | EncryptionAlgorithm::Rsa3072
+        | EncryptionAlgorithm::Rsa4096 => "rsa",
+        EncryptionAlgorithm::Ecc192
+        | EncryptionAlgorithm::Ecc224
+        | EncryptionAlgorithm::Ecc256
+        | EncryptionAlgorithm::Ecc384
+        | EncryptionAlgorithm::Ecc521
+        | EncryptionAlgorithm::EccSm2 => "ecc",
+    }
+}
+
 impl From<EncryptionAlgorithm> for AsymmetricAlgorithm {
     fn from(enc_alg: EncryptionAlgorithm) -> Self {
         match enc_alg {
@@ -270,16 +287,46 @@ impl TryFrom<&str> for EncryptionAlgorithm {
     }
 }
 
+/// API version 2.5 introduced specific algorithm names (rsa2048, ecc256)
+/// instead of generic names (rsa, ecc)
+const API_VERSION_2_5: crate::version::Version =
+    crate::version::Version::new(2, 5);
+
+impl EncryptionAlgorithm {
+    /// Format the encryption algorithm for a specific API version
+    ///
+    /// - For API versions < 2.5: Rsa2048 returns "rsa" and Ecc256 returns "ecc" (legacy format)
+    /// - For API versions >= 2.5: all algorithms use their specific format (e.g., "rsa2048", "ecc256")
+    pub fn to_string_for_api_version(
+        &self,
+        api_version: &crate::version::Version,
+    ) -> String {
+        // Only Rsa2048 and Ecc256 had different formats in legacy API versions
+        let (legacy_name, modern_name) = match self {
+            EncryptionAlgorithm::Rsa2048 => ("rsa", "rsa2048"),
+            EncryptionAlgorithm::Ecc256 => ("ecc", "ecc256"),
+            // All other algorithms use standard Display format regardless of version
+            _ => return self.to_string(),
+        };
+
+        if api_version < &API_VERSION_2_5 {
+            legacy_name.to_string()
+        } else {
+            modern_name.to_string()
+        }
+    }
+}
+
 impl fmt::Display for EncryptionAlgorithm {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         let value = match self {
             EncryptionAlgorithm::Rsa1024 => "rsa1024",
-            EncryptionAlgorithm::Rsa2048 => "rsa", /* for backwards compatibility */
+            EncryptionAlgorithm::Rsa2048 => "rsa2048",
             EncryptionAlgorithm::Rsa3072 => "rsa3072",
             EncryptionAlgorithm::Rsa4096 => "rsa4096",
             EncryptionAlgorithm::Ecc192 => "ecc192",
             EncryptionAlgorithm::Ecc224 => "ecc224",
-            EncryptionAlgorithm::Ecc256 => "ecc", /* for backwards compatibility */
+            EncryptionAlgorithm::Ecc256 => "ecc256",
             EncryptionAlgorithm::Ecc384 => "ecc384",
             EncryptionAlgorithm::Ecc521 => "ecc521",
             EncryptionAlgorithm::EccSm2 => "ecc_sm2",
@@ -485,6 +532,29 @@ mod tests {
         }
     } // test_get_key_size
 
+    #[test]
+    fn test_get_key_algorithm_family() {
+        let algorithms = [
+            (EncryptionAlgorithm::Rsa1024, "rsa"),
+            (EncryptionAlgorithm::Rsa2048, "rsa"),
+            (EncryptionAlgorithm::Rsa3072, "rsa"),
+            (EncryptionAlgorithm::Rsa4096, "rsa"),
+            (EncryptionAlgorithm::Ecc192, "ecc"),
+            (EncryptionAlgorithm::Ecc224, "ecc"),
+            (EncryptionAlgorithm::Ecc256, "ecc"),
+            (EncryptionAlgorithm::Ecc384, "ecc"),
+            (EncryptionAlgorithm::Ecc521, "ecc"),
+            (EncryptionAlgorithm::EccSm2, "ecc"),
+        ];
+        for (alg, expected_family) in algorithms {
+            let family = get_key_algorithm_family(&alg);
+            assert_eq!(
+                family, expected_family,
+                "Algorithm family mismatch for {alg}"
+            );
+        }
+    } // test_get_key_algorithm_family
+
     #[test]
     fn test_get_ecc_curve_key_size() {
         let curves = [
