@@ -75,7 +75,7 @@ impl<'a> CertificateBuilder<'a> {
     pub fn common_name(
         &'a mut self,
         cn: &'a str,
-    ) -> &mut CertificateBuilder<'a> {
+    ) -> &'a mut CertificateBuilder<'a> {
         self.common_name = Some(cn);
         self
     }
@@ -88,7 +88,7 @@ impl<'a> CertificateBuilder<'a> {
     pub fn hash_algorithm(
         &'a mut self,
         hash_algorithm: MessageDigest,
-    ) -> &mut CertificateBuilder<'a> {
+    ) -> &'a mut CertificateBuilder<'a> {
         self.hash_algorithm = Some(hash_algorithm);
         self
     }
@@ -98,11 +98,11 @@ impl<'a> CertificateBuilder<'a> {
     /// # Arguments:
     ///
     /// * days_from_now (u32): The number of days from now when the built certificate will become
-    /// valid
+    ///   valid
     pub fn not_before(
         &'a mut self,
         days_from_now: u32,
-    ) -> &mut CertificateBuilder<'a> {
+    ) -> &'a mut CertificateBuilder<'a> {
         self.not_before = Some(days_from_now);
         self
     }
@@ -115,7 +115,7 @@ impl<'a> CertificateBuilder<'a> {
     pub fn not_after(
         &'a mut self,
         days_from_now: u32,
-    ) -> &mut CertificateBuilder<'a> {
+    ) -> &'a mut CertificateBuilder<'a> {
         self.not_after = Some(days_from_now);
         self
     }
@@ -125,13 +125,13 @@ impl<'a> CertificateBuilder<'a> {
     /// # Arguments:
     ///
     /// * version (i32): The version number. Note that the version is zero-indexed, meaning passing
-    /// the value `2` corresponds to the version 3 of the X.509 standard
+    ///   the value `2` corresponds to the version 3 of the X.509 standard
     ///
     /// If not called, the version 3 of the X.509 standard will be used
     pub fn version(
         &'a mut self,
         version: i32,
-    ) -> &mut CertificateBuilder<'a> {
+    ) -> &'a mut CertificateBuilder<'a> {
         self.version = Some(version);
         self
     }
@@ -144,7 +144,7 @@ impl<'a> CertificateBuilder<'a> {
     pub fn private_key(
         &'a mut self,
         private_key: &'a PKey<Private>,
-    ) -> &mut CertificateBuilder<'a> {
+    ) -> &'a mut CertificateBuilder<'a> {
         self.private_key = Some(private_key);
         self
     }
@@ -154,11 +154,11 @@ impl<'a> CertificateBuilder<'a> {
     /// # Arguments:
     ///
     /// * dns_names (Vec<&str>): A Vec<&str> containing DNS names to add to the certificate Subject
-    /// Alternative Name
+    ///   Alternative Name
     pub fn add_dns_names(
         &'a mut self,
         dns_names: Vec<&'a str>,
-    ) -> &mut CertificateBuilder<'a> {
+    ) -> &'a mut CertificateBuilder<'a> {
         match &mut self.dns_names {
             None => {
                 self.dns_names = Some(dns_names);
@@ -177,11 +177,11 @@ impl<'a> CertificateBuilder<'a> {
     /// # Arguments:
     ///
     /// * ips: (Vec<&str>): A Vec<&str> containing IPs to add to the certificate Subject
-    /// Alternative Name
+    ///   Alternative Name
     pub fn add_ips(
         &'a mut self,
         ips: Vec<&'a str>,
-    ) -> &mut CertificateBuilder<'a> {
+    ) -> &'a mut CertificateBuilder<'a> {
         match &mut self.ips {
             None => {
                 self.ips = Some(ips);
@@ -200,11 +200,11 @@ impl<'a> CertificateBuilder<'a> {
     /// # Arguments:
     ///
     /// * extensions (Vec<X509Extension>): A Vec<X509Extension> containing the additional
-    /// extensions to include in the certificate
+    ///   extensions to include in the certificate
     pub fn add_extensions(
         &'a mut self,
         extensions: Vec<X509Extension>,
-    ) -> &mut CertificateBuilder<'a> {
+    ) -> &'a mut CertificateBuilder<'a> {
         match &mut self.extensions {
             None => {
                 self.extensions = Some(extensions);
