@@ -2,6 +2,7 @@ pub mod agent_data;
 pub mod agent_identity;
 pub mod agent_registration;
 pub mod algorithms;
+pub mod boot_time;
 pub mod cert;
 pub mod config;
 pub mod context_info;
@@ -19,6 +20,7 @@ pub mod list_parser;
 pub mod permissions;
 pub mod quote;
 pub mod registrar_client;
+pub mod resilient_client;
 pub mod secure_mount;
 pub mod serialization;
 pub mod structures;
