@@ -6,7 +6,7 @@ use serde::{Deserialize, Serialize};
 *      Request.  *
 ******************/
 #[derive(Serialize, Deserialize, Debug)]
-pub struct SessionRequestAuthSupported {
+pub struct SupportedAuthMethod {
     #[serde(rename(
         serialize = "authentication_class",
         deserialize = "authentication_class"
@@ -28,7 +28,7 @@ pub struct SessionRequestAttributes {
         serialize = "authentication_supported",
         deserialize = "authentication_supported"
     ))]
-    pub auth_supported: Vec<SessionRequestAuthSupported>,
+    pub auth_supported: Vec<SupportedAuthMethod>,
 }
 
 #[derive(Serialize, Deserialize, Debug)]
@@ -89,7 +89,7 @@ pub struct SessionResponseAttributes {
 pub struct SessionResponseData {
     #[serde(rename(serialize = "type", deserialize = "type"))]
     pub data_type: String,
-    pub id: u64,
+    pub id: String, // JSON:API requires IDs to be strings (can be UUID or numeric)
     pub attributes: SessionResponseAttributes,
 }
 
@@ -103,7 +103,7 @@ pub struct SessionResponse {
 *      Request.       *
 ***********************/
 #[derive(Serialize, Deserialize, Debug)]
-pub struct SessionIdRequestAuthProvidedData {
+pub struct ProofOfPossession {
     pub message: String,
     pub signature: String,
 }
@@ -122,11 +122,11 @@ pub struct SessionIdRequestAuthProvided {
     ))]
     pub auth_type: String,
 
-    pub data: SessionIdRequestAuthProvidedData,
+    pub data: ProofOfPossession,
 }
 
 #[derive(Serialize, Deserialize, Debug)]
-pub struct SessionIdRequestAtttributes {
+pub struct SessionUpdateAttributes {
     pub agent_id: String,
     #[serde(rename(
         serialize = "authentication_provided",
@@ -139,8 +139,8 @@ pub struct SessionIdRequestAtttributes {
 pub struct SessionIdRequestData {
     #[serde(rename(serialize = "type", deserialize = "type"))]
     data_type: String,
-    pub id: u64,
-    pub attributes: SessionIdRequestAtttributes,
+    pub id: String, // JSON:API requires IDs to be strings
+    pub attributes: SessionUpdateAttributes,
 }
 
 #[derive(Serialize, Deserialize, Debug)]
@@ -148,6 +148,19 @@ pub struct SessionIdRequest {
     pub data: SessionIdRequestData,
 }
 
+impl SessionIdRequest {
+    /// Create a new session ID request (for PATCH /sessions/:id)
+    pub fn new(id: String, attributes: SessionUpdateAttributes) -> Self {
+        Self {
+            data: SessionIdRequestData {
+                data_type: "session".to_string(),
+                id,
+                attributes,
+            },
+        }
+    }
+}
+
 /**********************
 * PATCH /sessions/:id *
 *      Response.      *
@@ -185,7 +198,7 @@ pub struct SessionIdResponseAuth {
 }
 
 #[derive(Serialize, Deserialize, Debug)]
-pub struct SessionIdResponseAtttributes {
+pub struct AuthenticationResultAttributes {
     pub agent_id: String,
     pub evaluation: String,
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -208,8 +221,8 @@ pub struct SessionIdResponseAtttributes {
 pub struct SessionIdResponseData {
     #[serde(rename(serialize = "type", deserialize = "type"))]
     data_type: String,
-    pub id: u64,
-    pub attributes: SessionIdResponseAtttributes,
+    pub id: String, // JSON:API requires IDs to be strings
+    pub attributes: AuthenticationResultAttributes,
 }
 
 #[derive(Serialize, Deserialize, Debug)]
@@ -229,7 +242,7 @@ mod tests {
                 data_type: "session".to_string(),
                 attributes: SessionRequestAttributes {
                     agent_id: "example-agent".to_string(),
-                    auth_supported: vec![SessionRequestAuthSupported {
+                    auth_supported: vec![SupportedAuthMethod {
                         auth_class: "pop".to_string(),
                         auth_type: "tpm_pop".to_string(),
                     }],
@@ -295,7 +308,7 @@ mod tests {
         let session_response = SessionResponse {
             data: SessionResponseData {
                 data_type: "session".to_string(),
-                id: 1,
+                id: "1".to_string(),
                 attributes: SessionResponseAttributes {
                     agent_id: "example-agent".to_string(),
                     auth_requested: vec![SessionResponseAuthRequested {
@@ -319,7 +332,7 @@ mod tests {
         let expected = r#"{
   "data": {
     "type": "session",
-    "id": 1,
+    "id": "1",
     "attributes": {
       "agent_id": "example-agent",
       "authentication_requested": [
@@ -344,7 +357,7 @@ mod tests {
         let json = r#"{
             "data": {
                 "type": "session",
-                "id": 1,
+                "id": "1",
                 "attributes": {
                     "agent_id": "example-deserialized-agent",
                     "authentication_requested": [
@@ -364,7 +377,7 @@ mod tests {
         let deserialized: SessionResponse =
             serde_json::from_str(json).unwrap(); //#[allow_ci]
         assert_eq!(deserialized.data.data_type, "session");
-        assert_eq!(deserialized.data.id, 1);
+        assert_eq!(deserialized.data.id, "1");
         assert_eq!(
             deserialized.data.attributes.agent_id,
             "example-deserialized-agent"
@@ -391,13 +404,13 @@ mod tests {
         let session_id_request = SessionIdRequest {
             data: SessionIdRequestData {
                 data_type: "session".to_string(),
-                id: 1,
-                attributes: SessionIdRequestAtttributes {
+                id: "1".to_string(),
+                attributes: SessionUpdateAttributes {
                     agent_id: "example-agent".to_string(),
                     auth_provided: vec![SessionIdRequestAuthProvided {
                         auth_class: "pop".to_string(),
                         auth_type: "tpm_pop".to_string(),
-                        data: SessionIdRequestAuthProvidedData {
+                        data: ProofOfPossession {
                             message: "example-message".to_string(),
                             signature: "example-signature".to_string(),
                         },
@@ -411,7 +424,7 @@ mod tests {
         let expected = r#"{
   "data": {
     "type": "session",
-    "id": 1,
+    "id": "1",
     "attributes": {
       "agent_id": "example-agent",
       "authentication_provided": [
@@ -435,7 +448,7 @@ mod tests {
         let json = r#"{
             "data": {
                 "type": "session",
-                "id": 1,
+                "id": "1",
                 "attributes": {
                     "agent_id": "example-deserialized-agent",
                     "authentication_provided": [
@@ -454,7 +467,7 @@ mod tests {
         let deserialized: SessionIdRequest =
             serde_json::from_str(json).unwrap(); //#[allow_ci]
         assert_eq!(deserialized.data.data_type, "session");
-        assert_eq!(deserialized.data.id, 1);
+        assert_eq!(deserialized.data.id, "1");
         assert_eq!(
             deserialized.data.attributes.agent_id,
             "example-deserialized-agent"
@@ -479,10 +492,10 @@ mod tests {
         let session_id_response = SessionIdResponse {
             data: SessionIdResponseData {
                 data_type: "session".to_string(),
-                id: 1,
-                attributes: SessionIdResponseAtttributes {
+                id: "1".to_string(),
+                attributes: AuthenticationResultAttributes {
                     agent_id: "example-agent".to_string(),
-                    evaluation: "success".to_string(),
+                    evaluation: "pass".to_string(),
                     token: Some("example-token".to_string()),
                     auth: vec![SessionIdResponseAuth {
                         auth_class: "pop".to_string(),
@@ -516,10 +529,10 @@ mod tests {
         let expected = r#"{
   "data": {
     "type": "session",
-    "id": 1,
+    "id": "1",
     "attributes": {
       "agent_id": "example-agent",
-      "evaluation": "success",
+      "evaluation": "pass",
       "token": "example-token",
       "authentication": [
         {
@@ -549,10 +562,10 @@ mod tests {
         let json = r#"{
             "data": {
                 "type": "session",
-                "id": 1,
+                "id": "1",
                 "attributes": {
                     "agent_id": "example-deserialized-agent",
-                    "evaluation": "success",
+                    "evaluation": "pass",
                     "token": "example-deserialized-token",
                     "authentication": [
                         {
@@ -577,12 +590,12 @@ mod tests {
         let deserialized: SessionIdResponse =
             serde_json::from_str(json).unwrap(); //#[allow_ci]
         assert_eq!(deserialized.data.data_type, "session");
-        assert_eq!(deserialized.data.id, 1);
+        assert_eq!(deserialized.data.id, "1");
         assert_eq!(
             deserialized.data.attributes.agent_id,
             "example-deserialized-agent"
         );
-        assert_eq!(deserialized.data.attributes.evaluation, "success");
+        assert_eq!(deserialized.data.attributes.evaluation, "pass");
         assert_eq!(
             deserialized.data.attributes.token,
             Some("example-deserialized-token".to_string())
@@ -627,8 +640,8 @@ mod tests {
         let session_id_response = SessionIdResponse {
             data: SessionIdResponseData {
                 data_type: "session".to_string(),
-                id: 1,
-                attributes: SessionIdResponseAtttributes {
+                id: "1".to_string(),
+                attributes: AuthenticationResultAttributes {
                     agent_id: "example-agent".to_string(),
                     evaluation: "fail".to_string(),
                     token: None,
@@ -659,7 +672,7 @@ mod tests {
         let expected = r#"{
   "data": {
     "type": "session",
-    "id": 1,
+    "id": "1",
     "attributes": {
       "agent_id": "example-agent",
       "evaluation": "fail",
